package juego;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Scanner;

import javax.swing.JOptionPane;

import servidor.Mensaje;
import stratego.Tablero;
import stratego.Movimiento;
import stratego.Pieza;
import stratego.Rango;
import stratego.Puntero;


public class Cliente
{
	private String ip = null;
	private Integer port = null;
	private Socket sock = null;
	private static DataInputStream in = null;
	private static DataOutputStream out = null;

	private static Vista view = null;
	
	public Cliente(Vista v)
	{
		view = v;
	}
	
	public boolean init()
	{
		ip = JOptionPane.showInputDialog("Introduzca IP para conectarse a:");
		if (ip == null) return false;
		try
		{
			port = Integer.parseInt(JOptionPane.showInputDialog("Introduzca el puerto para conectarse a:"));
		}
		catch (Exception e)
		{
			return false;
		}
		
		return connect();
	}
	
	public boolean restart()
	{
		if (ip == null || port == null)
			return false;
		return connect();
	}
	
	private boolean connect()
	{
		try {
			sock = new Socket(ip, port);
		} catch (UnknownHostException e)
		{
			JOptionPane.showMessageDialog(null, "Host no funciona",
					"Error de Conexin", JOptionPane.ERROR_MESSAGE);
			sock = null;
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, e.getMessage(),
					"Error de Conexin", JOptionPane.ERROR_MESSAGE);
			sock = null;
		}
		
		if (sock != null)
		{
			try {
				in = new DataInputStream(sock.getInputStream());
				out = new DataOutputStream(sock.getOutputStream());
				
				out.writeInt(Mensaje.PROTOCOL_VERSION);
				
				listen();
				return true;
			} catch (IOException e)
			{
				e.printStackTrace();
			}
		}
		return false;
	}
	
	private void listen()
	{
		new Thread() 
		{
			public void run()
			{
				while (true)
				{
					try
					{
						int x, y, color, rank;
						
						int type = in.readInt();
						switch (Mensaje.values()[type])
						{
						case SETUP:
							x = in.readInt();
							view.setColor(x);
							break;
						case GRID:
							x = in.readInt();
							y = in.readInt();
							color = in.readInt();
							rank = in.readInt();
							view.update(new Puntero(x, y), new Pieza(0, color, Rango.values()[rank]));
							break;
						case TRAY:
							x = in.readInt();
							color = in.readInt();
							rank = in.readInt();
							view.update(Tablero.IN_TRAY, new Pieza(x, color, Rango.values()[rank]));
							break;
						case GAMEOVER:
							x = in.readInt();
							view.gameOver(x);
							return;
						case BYE:
							view.gameOver(-1);
							return;
						case UPDATE:
							String downloadURL = new Scanner(in).next();
							JOptionPane.showMessageDialog(null, "Tu versin de Stratego es incompatible con esta versin.\n" +
									"Por favor descargue la versin mas reciente en " + downloadURL, "Stratego", JOptionPane.ERROR_MESSAGE);
							view.gameOver(-1);
							return;
						default:
							break;
						}
					}
					catch (IOException e)
					{
						view.gameOver(-1);
						return;
					}
				}
			}
		}.start();
	}
	
	public void disconnect()
	{
		try
		{
			out.writeInt(Mensaje.BYE.ordinal());
			out.flush();
		}
		catch (IOException e){}
		finally
		{
			try {
				sock.close();
			} catch (IOException e){}
		}
	}

	public  boolean move(Movimiento m)
	{
		try
		{
			out.writeInt(Mensaje.MOVE.ordinal());

			out.writeInt(m.getFrom().getX());
			out.writeInt(m.getFrom().getY());
			out.writeInt(m.getTo().getX());
			out.writeInt(m.getTo().getY());
			out.writeInt(m.getPiece().getRank().ordinal());
			
			out.flush();
		}
		catch (IOException e)
		{
			return false;
		}
		return true;
	}

	public void play()
	{
		try
		{
			out.writeInt(Mensaje.PLAY.ordinal());
			out.flush();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}

}
