package juego.editar;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;

import juego.BotonesPiezas;
import juego.Vista;


import stratego.Movimiento;
import stratego.Puntero;



public class Editar extends Vista
{
	private EditarControles board = null;  //  @jve:decl-index=0:
	private JButton openButton = null;
	private JButton saveButton = null;
	private JButton saveAsButton = null;
	
	private String fn = "";
	
	public Editar(String f)
	{
		fn = f;
		unSplash();
	}

	public void registerEngine(EditarControles b) 
	{
		board = b;
		update();
		getJFrame().setTitle("Editar Estrategia");
		
		if (!fn.equals(""))
			try {
				open();
			} catch (IOException e) {
				e.printStackTrace();
			}
	}
	
	private void open() throws IOException
	{
	    if (fn.equals(""))
	    {
	    	openAs();
	    	return;
	    }
	    
		BufferedReader in = new BufferedReader(new FileReader(fn));
		
		board.clear();
		
		try
		{
			for (int j=0;j<40;j++)
			{
				int x = in.read(),
					y = in.read();
				
				if (x<0||x>9||y<0||y>3)
					throw new Exception();
				
				board.move(new Movimiento(board.getTrayPiece(0), EditarTablero.IN_TRAY, new Puntero(x, y)));
			}
		}
		catch (IOException e)
		{
			JOptionPane.showMessageDialog(getJFrame(), "Error de formato de archivo : Archivo inesperado.", 
					"Editar Estrategia", JOptionPane.INFORMATION_MESSAGE);
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(getJFrame(), "Error de formato de archivo: Estructura de archivo no vlida.", 
					"Editar Estrategia", JOptionPane.INFORMATION_MESSAGE);
		}
		finally
		{
			try {
				in.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
			
			update();
		}
	}
	
	protected void openAs() throws IOException
	{
	    JFileChooser chooser = new JFileChooser();
	    int returnVal = chooser.showOpenDialog(this.getJFrame());
	    if(returnVal != JFileChooser.APPROVE_OPTION)
	    	return;

	    fn = chooser.getSelectedFile().getCanonicalPath();
	    
	    open();
	}	
	
	private void save() throws IOException
	{
		if (fn.equals(""))
		{
			saveAs();
			return;
		}
		
		for (int j=0;j<4;j++)
		for (int i=0;i<10;i++)
			if (board.getPiece(i, j) == null) 
			{

				JOptionPane.showMessageDialog(getJFrame(), "Configuracin del tablero incompleta.",
						"Editar Estrategia", JOptionPane.INFORMATION_MESSAGE);
				return;
			}

		ArrayList<BotonesPiezas> setup = new ArrayList<BotonesPiezas>();
		
		for (int j=0;j<4;j++)
		for (int i=0;i<10;i++)
			setup.add(grid[i][j]);
		Collections.sort(setup);
		

		BufferedWriter out = new BufferedWriter(new FileWriter(fn));
		
		for (BotonesPiezas p: setup)
		{
			out.write(p.getSpot().getX());
			out.write(p.getSpot().getY());
		}
		
		out.close();
	}
	
	protected void saveAs() throws IOException
	{
	    JFileChooser chooser = new JFileChooser();
	    int returnVal = chooser.showSaveDialog(this.getJFrame());
	    if(returnVal != JFileChooser.APPROVE_OPTION)
	    	return;

	    fn = chooser.getSelectedFile().getCanonicalPath();
	    
	    save();
	}

	public void update()
	{		
		for (int i=0;i<10;i++)
		for (int j=0;j<10;j++)
			grid[i][j].setPiece(board.getPiece(i, j));

		for (int i=0;i<40;i++)
			trayComp[i].setPiece(null);
		
		for (int i=0;i<board.getTraySize();i++)
			{
				trayComp[i].setPiece(board.getTrayPiece(i));
			}
	}
	
	public void moveAction(Movimiento m)
	{
		board.move(m);
		update();
	}
	

	protected void skinToolBar()
	{
		getOpenButton().setIcon(skin.scaledOpenIcon);
		getSaveButton().setIcon(skin.scaledSaveIcon);
		getSaveAsButton().setIcon(skin.scaledSaveAsIcon);
	}
	
	protected JMenuBar getJToolBar() {
		if (jToolBar == null) {
			jToolBar = new JMenuBar();
			jToolBar.add(getOpenButton());
			jToolBar.add(getSaveButton());
			jToolBar.add(getSaveAsButton());
		}
		return jToolBar;
	}

	private JButton getOpenButton()
	{
		if (openButton == null)
		{
			openButton = new JButton();
			openButton.setText("Abrir");
			openButton.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
						try {
							openAs();
						} catch (IOException e1) {
							e1.printStackTrace();
						}
				}
			});
		}
		return openButton;
	}

	private JButton getSaveButton()
	{
		if (saveButton == null)
		{
			saveButton = new JButton();
			saveButton.setText("Guardar");
			saveButton.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					try {
						save();
					} catch (IOException e1) {
						e1.printStackTrace();
					}
				}
			});
		}
		return saveButton;
	}

	private JButton getSaveAsButton()
	{
		if (saveAsButton == null)
		{
			saveAsButton = new JButton();
			saveAsButton.setText("Guardar en");
			saveAsButton.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					try {
						saveAs();
					} catch (IOException e1) {
						e1.printStackTrace();
					}
				}
			});
		}
		return saveAsButton;
	}

	protected void skinToolBar(int w, int h) {};
}
